@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -20,61 +22,13 @@
 
 
 template<typename eT>
-arma_hot
 arma_inline
 void
 arrayops::copy(eT* dest, const eT* src, const uword n_elem)
   {
-  if(dest != src)
-    {
-    if(is_cx<eT>::no)
-      {
-      if(n_elem <= 9)
-        {
-        arrayops::copy_small(dest, src, n_elem);
-        }
-      else
-        {
-        std::memcpy(dest, src, n_elem*sizeof(eT));
-        }
-      }
-    else
-      {
-      if(n_elem > 0)  { std::memcpy(dest, src, n_elem*sizeof(eT)); }
-      }
-    }
-  }
-
-
-
-template<typename eT>
-arma_cold
-inline
-void
-arrayops::copy_small(eT* dest, const eT* src, const uword n_elem)
-  {
-  switch(n_elem)
-    {
-    case  9:  dest[ 8] = src[ 8];
-    // fallthrough
-    case  8:  dest[ 7] = src[ 7];
-    // fallthrough
-    case  7:  dest[ 6] = src[ 6];
-    // fallthrough
-    case  6:  dest[ 5] = src[ 5];
-    // fallthrough
-    case  5:  dest[ 4] = src[ 4];
-    // fallthrough
-    case  4:  dest[ 3] = src[ 3];
-    // fallthrough
-    case  3:  dest[ 2] = src[ 2];
-    // fallthrough
-    case  2:  dest[ 1] = src[ 1];
-    // fallthrough
-    case  1:  dest[ 0] = src[ 0];
-    // fallthrough
-    default:  ;
-    }
+  if( (n_elem == 0) || (dest == src) )  { return; }
+  
+  std::memcpy(dest, src, n_elem*sizeof(eT));
   }
 
 
@@ -86,9 +40,11 @@ arrayops::fill_zeros(eT* dest, const uword n_elem)
   {
   typedef typename get_pod_type<eT>::result pod_type;
   
+  if( (n_elem == 0) || (dest == nullptr) )  { return; }
+  
   if(std::numeric_limits<eT>::is_integer || std::numeric_limits<pod_type>::is_iec559)
     {
-    if(n_elem > 0)  { std::memset((void*)dest, 0, sizeof(eT)*n_elem); }
+    std::memset((void*)dest, 0, sizeof(eT)*n_elem);
     }
   else
     {
@@ -175,8 +131,53 @@ arrayops::clean(std::complex<T>* mem, const uword n_elem, const T abs_limit)
 
 
 
+template<typename eT>
+inline
+void
+arrayops::clamp(eT* mem, const uword n_elem, const eT min_val, const eT max_val, const typename arma_not_cx<eT>::result* junk)
+  {
+  arma_ignore(junk);
+  
+  for(uword i=0; i<n_elem; ++i)
+    {
+    eT& val = mem[i];
+    
+    val = (val < min_val) ? min_val : ((val > max_val) ? max_val : val);
+    }
+  }
+
+
+
+template<typename T>
+inline
+void
+arrayops::clamp(std::complex<T>* mem, const uword n_elem, const std::complex<T>& min_val, const std::complex<T>& max_val)
+  {
+  typedef typename std::complex<T> eT;
+  
+  const T min_val_real = std::real(min_val);
+  const T min_val_imag = std::imag(min_val);
+  
+  const T max_val_real = std::real(max_val);
+  const T max_val_imag = std::imag(max_val);
+  
+  for(uword i=0; i<n_elem; ++i)
+    {
+    eT& val = mem[i];
+    
+    T val_real = std::real(val);
+    T val_imag = std::imag(val);
+    
+    val_real = (val_real < min_val_real) ? min_val_real : ((val_real > max_val_real) ? max_val_real : val_real);
+    val_imag = (val_imag < min_val_imag) ? min_val_imag : ((val_imag > max_val_imag) ? max_val_imag : val_imag);
+    
+    val = std::complex<T>(val_real,val_imag);
+    }
+  }
+
+
+
 template<typename out_eT, typename in_eT>
-arma_hot
 arma_inline
 void
 arrayops::convert_cx_scalar
@@ -196,7 +197,6 @@ arrayops::convert_cx_scalar
 
 
 template<typename out_eT, typename in_T>
-arma_hot
 arma_inline
 void
 arrayops::convert_cx_scalar
@@ -208,13 +208,16 @@ arrayops::convert_cx_scalar
   {
   arma_ignore(junk);
   
-  out = out_eT( in.real() );
+  const in_T val = in.real();
+  
+  const bool conversion_ok = (std::is_integral<out_eT>::value && std::is_floating_point<in_T>::value) ? arma_isfinite(val) : true;
+  
+  out = conversion_ok ? out_eT(val) : out_eT(0);
   }
 
 
 
 template<typename out_T, typename in_T>
-arma_hot
 arma_inline
 void
 arrayops::convert_cx_scalar
@@ -245,6 +248,7 @@ arrayops::convert(out_eT* dest, const in_eT* src, const uword n_elem)
     return;
     }
   
+  const bool check_finite = (std::is_integral<out_eT>::value && std::is_floating_point<in_eT>::value);
   
   uword j;
   
@@ -256,15 +260,26 @@ arrayops::convert(out_eT* dest, const in_eT* src, const uword n_elem)
     // dest[i] = out_eT( tmp_i );
     // dest[j] = out_eT( tmp_j );
     
-    (*dest) = (is_signed<out_eT>::value)
-              ? out_eT( tmp_i )
-              : ( cond_rel< is_signed<in_eT>::value >::lt(tmp_i, in_eT(0)) ? out_eT(0) : out_eT(tmp_i) );
+    const bool ok_i = check_finite ? arma_isfinite(tmp_i) : true;
+    const bool ok_j = check_finite ? arma_isfinite(tmp_j) : true;
+    
+    (*dest) = ok_i
+              ? (
+                (is_signed<out_eT>::value)
+                ? out_eT( tmp_i )
+                : ( cond_rel< is_signed<in_eT>::value >::lt(tmp_i, in_eT(0)) ? out_eT(0) : out_eT(tmp_i) )
+                )
+              : out_eT(0);
     
     dest++;
     
-    (*dest) = (is_signed<out_eT>::value)
-              ? out_eT( tmp_j )
-              : ( cond_rel< is_signed<in_eT>::value >::lt(tmp_j, in_eT(0)) ? out_eT(0) : out_eT(tmp_j) );
+    (*dest) = ok_j
+              ? (
+                (is_signed<out_eT>::value)
+                ? out_eT( tmp_j )
+                : ( cond_rel< is_signed<in_eT>::value >::lt(tmp_j, in_eT(0)) ? out_eT(0) : out_eT(tmp_j) )
+                )
+              : out_eT(0);
     dest++;
     }
   
@@ -274,9 +289,15 @@ arrayops::convert(out_eT* dest, const in_eT* src, const uword n_elem)
     
     // dest[i] = out_eT( tmp_i );
     
-    (*dest) = (is_signed<out_eT>::value)
-              ? out_eT( tmp_i )
-              : ( cond_rel< is_signed<in_eT>::value >::lt(tmp_i, in_eT(0)) ? out_eT(0) : out_eT(tmp_i) );
+    const bool ok_i = check_finite ? arma_isfinite(tmp_i) : true;
+    
+    (*dest) = ok_i
+              ? (
+                (is_signed<out_eT>::value)
+                ? out_eT( tmp_i )
+                : ( cond_rel< is_signed<in_eT>::value >::lt(tmp_i, in_eT(0)) ? out_eT(0) : out_eT(tmp_i) )
+                )
+              : out_eT(0);
     }
   }
 
@@ -612,14 +633,7 @@ arrayops::inplace_set(eT* dest, const eT val, const uword n_elem)
     }
   else
     {
-    if( (n_elem <= 9) && (is_cx<eT>::no) )
-      {
-      arrayops::inplace_set_small(dest, val, n_elem);
-      }
-    else
-      {
-      arrayops::inplace_set_simple(dest, val, n_elem);
-      }
+    arrayops::inplace_set_simple(dest, val, n_elem);
     }
   }
 
@@ -678,38 +692,6 @@ arrayops::inplace_set_base(eT* dest, const eT val, const uword n_elem)
 
 
 
-template<typename eT>
-arma_cold
-inline
-void
-arrayops::inplace_set_small(eT* dest, const eT val, const uword n_elem)
-  {
-  switch(n_elem)
-    {
-    case  9: dest[ 8] = val;
-    // fallthrough
-    case  8: dest[ 7] = val;
-    // fallthrough
-    case  7: dest[ 6] = val;
-    // fallthrough
-    case  6: dest[ 5] = val;
-    // fallthrough
-    case  5: dest[ 4] = val;
-    // fallthrough
-    case  4: dest[ 3] = val;
-    // fallthrough
-    case  3: dest[ 2] = val;
-    // fallthrough
-    case  2: dest[ 1] = val;
-    // fallthrough
-    case  1: dest[ 0] = val;
-    // fallthrough
-    default:;
-    }
-  }
-
-
-
 template<typename eT, const uword n_elem>
 arma_hot
 inline
@@ -1089,18 +1071,13 @@ arrayops::is_finite(const eT* src, const uword n_elem)
     const eT val_i = (*src);  src++;
     const eT val_j = (*src);  src++;
     
-    if( (arma_isfinite(val_i) == false) || (arma_isfinite(val_j) == false) )
-      {
-      return false;
-      }
+    if(arma_isfinite(val_i) == false)  { return false; }
+    if(arma_isfinite(val_j) == false)  { return false; }
     }
   
   if((j-1) < n_elem)
     {
-    if(arma_isfinite(*src) == false)
-      {
-      return false;
-      }
+    if(arma_isfinite(*src) == false)  { return false; }
     }
   
   return true;
